@@ -5,7 +5,7 @@ from pathlib import PurePath
 try:
     import tomllib as toml
 except ModuleNotFoundError:
-    import tomli as toml
+    import tomli as toml  # type: ignore
 
 with open(PurePath(__file__).parent / "pyproject.toml", "rb") as f:
     version = toml.load(f)["project"]["version"]
